@@ -8,11 +8,12 @@ except ImportError:
 
 import requests
 import requests.exceptions
+from requests_toolbelt.adapters import socket_options
 
 import marathon
 from .models import MarathonApp, MarathonDeployment, MarathonGroup, MarathonInfo, MarathonTask, MarathonEndpoint, MarathonQueueItem
-from .exceptions import InternalServerError, NotFoundError, MarathonHttpError, MarathonError
-from .models.events import EventFactory
+from .exceptions import ConflictError, InternalServerError, NotFoundError, MarathonHttpError, MarathonError, NoResponseError
+from .models.events import EventFactory, MarathonEvent
 from .util import MarathonJsonEncoder, MarathonMinimalJsonEncoder
 
 
@@ -21,7 +22,7 @@ class MarathonClient(object):
     """Client interface for the Marathon REST API."""
 
     def __init__(self, servers, username=None, password=None, timeout=10, session=None,
-                 auth_token=None, verify=True):
+                 auth_token=None, verify=True, sse_session=None):
         """Create a MarathonClient instance.
 
         If multiple servers are specified, each will be tried in succession until a non-"Connection Error"-type
@@ -36,11 +37,19 @@ class MarathonClient(object):
         :param int timeout: Timeout (in seconds) for requests to Marathon
         :param str auth_token: Token-based auth token, used with DCOS + Oauth
         :param bool verify: Enable SSL certificate verification
+        :param requests.session sse_session: requests.session for event stream connections, which by default enables tcp keepalive
         """
         if session is None:
             self.session = requests.Session()
         else:
             self.session = session
+        if sse_session is None:
+            self.sse_session = requests.Session()
+            keep_alive = socket_options.TCPKeepAliveAdapter()
+            self.sse_session.mount('http://', keep_alive)
+            self.sse_session.mount('https://', keep_alive)
+        else:
+            self.sse_session = sse_session
         self.servers = servers if isinstance(servers, list) else [servers]
         self.auth = (username, password) if username and password else None
         self.verify = verify
@@ -87,7 +96,7 @@ class MarathonClient(object):
                     'Error while calling %s: %s', url, str(e))
 
         if response is None:
-            raise MarathonError('No remaining Marathon servers to try')
+            raise NoResponseError('No remaining Marathon servers to try')
 
         if response.status_code >= 500:
             marathon.log.error('Got HTTP {code}: {body}'.format(
@@ -98,6 +107,8 @@ class MarathonClient(object):
                 code=response.status_code, body=response.text.encode('utf-8')))
             if response.status_code == 404:
                 raise NotFoundError(response)
+            elif response.status_code == 409:
+                raise ConflictError(response)
             else:
                 raise MarathonHttpError(response)
         elif response.status_code >= 300:
@@ -109,19 +120,22 @@ class MarathonClient(object):
 
         return response
 
-    def _do_sse_request(self, path):
+    def _do_sse_request(self, path, params=None):
         """Query Marathon server for events."""
         for server in list(self.servers):
             url = ''.join([server.rstrip('/'), path])
             try:
-                response = requests.get(
+                response = self.sse_session.get(
                     url,
+                    params=params,
                     stream=True,
                     headers={'Accept': 'text/event-stream'},
-                    auth=self.auth
+                    auth=self.auth,
+                    verify=self.verify,
                 )
             except Exception as e:
-                marathon.log.error('Error while calling %s: %s', url, e.message)
+                marathon.log.error(
+                    'Error while calling %s: %s', url, e.message)
             else:
                 if response.ok:
                     return response.iter_lines()
@@ -136,17 +150,18 @@ class MarathonClient(object):
         """
         return MarathonEndpoint.from_tasks(self.list_tasks())
 
-    def create_app(self, app_id, app):
+    def create_app(self, app_id, app, minimal=True):
         """Create and start an app.
 
         :param str app_id: application ID
         :param :class:`marathon.models.app.MarathonApp` app: the application to create
+        :param bool minimal: ignore nulls and empty collections
 
         :returns: the created app (on success)
         :rtype: :class:`marathon.models.app.MarathonApp` or False
         """
         app.id = app_id
-        data = app.to_json()
+        data = app.to_json(minimal=minimal)
         response = self._do_request('POST', '/v2/apps', data=data)
         if response.status_code == 201:
             return self._parse_response(response, MarathonApp)
@@ -156,7 +171,7 @@ class MarathonClient(object):
     def list_apps(self, cmd=None, embed_tasks=False, embed_counts=False,
                   embed_deployments=False, embed_readiness=False,
                   embed_last_task_failure=False, embed_failures=False,
-                  embed_task_stats=False, app_id=None, **kwargs):
+                  embed_task_stats=False, app_id=None, label=None, **kwargs):
         """List all apps.
 
         :param str cmd: if passed, only show apps with a matching `cmd`
@@ -168,6 +183,7 @@ class MarathonClient(object):
         :param bool embed_failures: shorthand for embed_last_task_failure
         :param bool embed_task_stats: embed task stats in result
         :param str app_id: if passed, only show apps with an 'id' that matches or contains this value
+        :param str label: if passed, only show apps with the selected labels
         :param kwargs: arbitrary search filters
 
         :returns: list of applications
@@ -178,6 +194,8 @@ class MarathonClient(object):
             params['cmd'] = cmd
         if app_id:
             params['id'] = app_id
+        if label:
+            params['label'] = label
 
         embed_params = {
             'app.tasks': embed_tasks,
@@ -437,7 +455,8 @@ class MarathonClient(object):
         params = {'force': force}
         response = self._do_request(
             'PUT',
-            '/v2/groups/{group_id}/versions/{version}'.format(group_id=group_id, version=version),
+            '/v2/groups/{group_id}/versions/{version}'.format(
+                group_id=group_id, version=version),
             params=params)
         return response.json()
 
@@ -464,9 +483,9 @@ class MarathonClient(object):
         :returns: a dict containing the deployment id and version
         :rtype: dict
         """
-        params = {'scaleBy': scale_by}
+        data = {'scaleBy': scale_by}
         response = self._do_request(
-            'PUT', '/v2/groups/{group_id}'.format(group_id=group_id), params=params)
+            'PUT', '/v2/groups/{group_id}'.format(group_id=group_id), data=json.dumps(data))
         return response.json()
 
     def list_tasks(self, app_id=None, **kwargs):
@@ -658,13 +677,17 @@ class MarathonClient(object):
         response = self._do_request('GET', '/v2/deployments')
         return self._parse_response(response, MarathonDeployment, is_list=True)
 
-    def list_queue(self):
+    def list_queue(self, embed_last_unused_offers=False):
         """List all the tasks queued up or waiting to be scheduled.
 
         :returns: list of queue items
         :rtype: list[:class:`marathon.models.queue.MarathonQueueItem`]
         """
-        response = self._do_request('GET', '/v2/queue')
+        if embed_last_unused_offers:
+            params = {'embed': 'lastUnusedOffers'}
+        else:
+            params = {}
+        response = self._do_request('GET', '/v2/queue', params=params)
         return self._parse_response(response, MarathonQueueItem, is_list=True, resource_name='queue')
 
     def delete_deployment(self, deployment_id, force=False):
@@ -688,6 +711,11 @@ class MarathonClient(object):
                 'DELETE', '/v2/deployments/{deployment}'.format(deployment=deployment_id))
             return response.json()
 
+    def reset_delay(self, app_id):
+        self._do_request(
+            "DELETE", '/v2/queue/{app_id}/delay'.format(app_id=app_id)
+        )
+
     def get_info(self):
         """Get server configuration information.
 
@@ -733,17 +761,27 @@ class MarathonClient(object):
         response = self._do_request('GET', '/metrics')
         return response.json()
 
-    def event_stream(self, raw=False):
+    def event_stream(self, raw=False, event_types=None):
         """Polls event bus using /v2/events
 
         :param bool raw: if true, yield raw event text, else yield MarathonEvent object
+        :param event_types: a list of event types to consume
+        :type event_types: list[type] or list[str]
         :returns: iterator with events
         :rtype: iterator
         """
 
         ef = EventFactory()
 
-        for raw_message in self._do_sse_request('/v2/events'):
+        params = {
+            'event_type': [
+                EventFactory.class_to_event[et] if isinstance(
+                    et, type) and issubclass(et, MarathonEvent) else et
+                for et in event_types or []
+            ]
+        }
+
+        for raw_message in self._do_sse_request('/v2/events', params=params):
             try:
                 _data = raw_message.decode('utf8').split(':', 1)
 
